
/* Apex coordinate conversion file processing program. The program can accept
the input parameters from a user specified file. Then the APEX fortran subroutines 
are called to compute the magnetic fields. The results are then printed to the specified
output file.

The WMM sublibrary, apex.f, apexsh.f90, makeapexsh.f90, magfld.f are all used in this program. The program expects the EGM9615.BIN to
 be in the same directory.

The program uses the user interface developed for geomag61.c
Note the option for geocentric height (C) is not supported in this version
The height entered is considered as height above mean sea level

Manoj.C.Nair@Noaa.Gov
October 20, 2011


*/
/****************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include <math.h>               /* for gcc */

#include "GeomagnetismLibrary.c"

#define NaN log(-1.0)
/* constants */
#define RECL 81

#define MAXINBUFF RECL+14

/** Max size of in buffer **/

#define MAXREAD MAXINBUFF-2
/** Max to read 2 less than total size (just to be safe) **/


#define PATH MAXREAD

#define QD_ALT_C 108
#define APEX_MLAT 361
#define APEX_MLON 601
#define APEX_MALT 17
#define APEX_NWK_C (APEX_MLAT * APEX_MLON * APEX_MALT * 5 + APEX+MLAT + APEX_MLON + APEX_MALT)

extern void makeapxsh_(char *datafilein, float *epochgridin, int *nepochin, int *lmax, int *mmaxin, int *nnmaxin);
extern void apxg2q_(float *glat, float *glon, float *alt, int *vecflag, float *qlat, float *qlon, float *f1, float *f2, float *f);
extern void loadapxsh_(char *datafile, float *epoch);
extern void apxg2all_(float *glat, float *glon, float *alt, float *hr, int *vecflag, float *qlat, float *qlon, float *mlat, float *mlon, float *f1, float *f2, float *f, float *d1, float *d2, float *d3, float *d, float *e1, float *e2, float *e3);


/****************************************************************************/
/*                                                                          */
/*      Some variables used in this program                                 */
/*                                                                          */
/*    Name         Type                    Usage                            */
/* ------------------------------------------------------------------------ */

/*                                                                          */
/*   minalt     double array of MAXMOD  Minimum height of model.             */
/*                                                                          */
/*   altmin     double                  Minimum height of selected model.    */
/*                                                                          */
/*   altmax     double array of MAXMOD  Maximum height of model.             */
/*                                                                          */
/*   maxalt     double                  Maximum height of selected model.    */
/*                                                                          */
/*   sdate  Scalar double           start date inputted                      */
/*                                                                          */
/*   alt        Scalar double           altitude above WGS84 Ellipsoid       */
/*                                                                          */
/*   latitude   Scalar double           Latitude.                            */
/*                                                                          */
/*   longitude  Scalar double           Longitude.                           */
/*                                                                          */
/*   inbuff     Char a of MAXINBUF     Input buffer.                        */
/*                                                                          */
/*                                                                          */
/*   minyr      double                  Min year of all models               */
/*                                                                          */
/*   maxyr      double                  Max year of all models               */
/*                                                                          */
/*   yrmax      double array of MAXMOD  Max year of model.                   */
/*                                                                          */
/*   yrmin      double array of MAXMOD  Min year of model.                   */
/*                                                                          */
/****************************************************************************/


int main(int argv, char**argc)
{
#ifdef MAC
  ccommand(argv, argc);
#endif
  /*  WMM Variable declaration  */

	MAGtype_Ellipsoid Ellip;
	MAGtype_CoordGeodetic CoordGeodetic;
	MAGtype_CoordSpherical CoordSpherical;
	float maglat, maglon, qlat, qlon;
	MAGtype_Date UserDate;
	MAGtype_Geoid Geoid;
	char ans[20];
	//char filename[] = "IGRF.tab";
	int nepochgrid = 5;
	float epochgrid[nepochgrid], epoch;
	float ra1 = 0, ra2 = 0, ra3 = 0, ra4 = 0, ra5 = 0, ra6 = 0, ra7 = 0, ra8 = 0, ra9 = 0, ra10 = 0, apex_alt, flat, flon, falt;
	int lmax = 3, nmmax = 6, vecflag = 0;


  /* Control variables */
  int   again = 1;
  int   decyears = 3;
  int   units = 4;
  int   decdeg = 3;
  int   range = -1;
  int   igdgc=3;
  int   isyear=-1;
  int   ismonth=-1;
  int   isday=-1;
  int   ieyear=-1;
  int   iemonth=-1;
  int   ieday=-1;
  int   ilat_deg=200;
  int   ilat_min=200;
  int   ilat_sec=200;
  int   ilon_deg=200;
  int   ilon_min=200;
  int   ilon_sec=200;


  int  coords_from_file = 0;
  int arg_err = 0;


  char  inbuff[MAXINBUFF];

  char *begin;
  char *rest;
  char args[7][MAXREAD];
  int iarg;

  char coord_fname[PATH];
  char out_fname[PATH];
  FILE *coordfile,*outfile;
  int iline=0;
  int read_flag;

  double minyr = 1900.0;
  double maxyr = 2015.0;
  double minalt;
  double maxalt;
  double alt=-999999;
  double sdate=-1;
  double step=-1;
  double edate=-1;
  double latitude=200;
  double longitude=200;



  /*  Subroutines used  */
  
  void print_result_file(FILE *outf, double maglat, double maglon);
  double degrees_to_decimal();
  double julday();
  int   safegets(char *buffer,int n);
  int getshc();


  /* Initializations. */

  inbuff[MAXREAD+1]='\0';  /* Just to protect mem. */
  inbuff[MAXINBUFF-1]='\0';  /* Just to protect mem. */


 /* Memory allocation */
	
//	MAG_SetDefaults(&Ellip, NULL, NULL); /* Set default values and constants */
	Ellip.a	=			6378.137; /*semi-major axis of the ellipsoid in */
	Ellip.b	=			6356.7523142;/*semi-minor axis of the ellipsoid in */
	Ellip.fla	=			1/298.257223563;/* flattening */
	Ellip.eps	=			sqrt(1- ( Ellip.b *	Ellip.b) / (Ellip.a * Ellip.a ));  /*first eccentricity */
	Ellip.epssq	=			(Ellip.eps * Ellip.eps);   /*first eccentricity squared */
	Ellip.re	=			6371.2;/* Earth's radius */
	epochgrid[0] = 1990;
	epochgrid[1] = 1995;
	epochgrid[2] = 2000;
	epochgrid[3] = 2005;
	epochgrid[4] = 2010;

	makeapxsh_("apexsh.dat", epochgrid, &nepochgrid, &lmax, &nmmax, &nmmax);

//	MAG_InitializeGeoid(&Geoid);    /* Read the Geoid file */

  for (iarg=0; iarg<argv; iarg++)
	if (argc[iarg] != NULL)
	  strncpy(args[iarg],argc[iarg],MAXREAD);

  //printing out version number and header
  printf("\n\n Apex Coordinate File processing program October 21, 2011 ");

  if (argv==1 || ((argv==2)&&(*(args[1])=='h')))
	{
	  printf("\n\nApex Coordinate - File Processing Utility : USAGE:\n");
	  printf("coordinate file: apx_file f input_file output_file\n");
	  printf("or for help:     apx_file h \n");
	  printf("\n");
	  printf("The input file must have the following entries in\n");
	  printf("the following format\n");
	  printf("Date and location Formats: \n");
	  printf("   Date: xxxx.xxx for decimal  (2013.7)\n");
	  printf("   Lat/Lon: xxx.xxx in decimal  (-76.53)\n");
	  printf("            (Lat and Lon must be specified in the same format.)\n");
	  printf("   Date and altitude must fit model.\n");
	  printf("   Lat: -90 to 90 (Use - to denote Southern latitude.)\n");
	  printf("   Lon: -180 to 180 (Use - to denote Western longitude.)\n");
	  printf("   Date: 1990.0 to 2015.0\n");
	  printf("   An example of an entry in input file\n");
	  printf("   2013.7 -70.3 -30.8 \n");
		printf("\n Press enter to exit.");
		printf ("\n >");
		fgets(ans, 20, stdin);

// 	if (Geoid.GeoidHeightBuffer)
// 	{
// 		free(Geoid.GeoidHeightBuffer);
// 		Geoid.GeoidHeightBuffer = NULL;
// 	}
	exit(2);
	} /* help */
  
  if ((argv==4)&&(*(args[1])=='f'))
    {
	  printf("\n\n 'f' switch: converting file with multiple locations.\n");
	  printf("     The first three output columns repeat the input coordinates.\n");
	  printf("     Then follows the Geomagnetic Latitude and Longitude.\n");

	  coords_from_file = 1;
	  strncpy(coord_fname,args[2],MAXREAD);
	  coordfile=fopen(coord_fname, "rt");
      strncpy(out_fname,args[3],MAXREAD);
      outfile=fopen(out_fname, "w");
      fprintf(outfile,"Date Latitude Longitude Geomagnetic-Latitude Geomagnetic-Longitude\n");
	} /* file option */

  if (argv>=2 && argv !=4)
	{
	  printf("\n\nERROR in 'f' switch option: wrong number of arguments\n");
	  exit(2);
	}

  if (argv==1)
	{
	  printf("\n\nERROR in switch option: wrong number of arguments\n");
	  exit(2);
	}



   while (again == 1)
	{
      if (coords_from_file) 
		{
		  argv = 6;
		  read_flag = fscanf(coordfile,"%s%s%s%*[^\n]",args[1],args[4],args[5]);
		  if (read_flag == EOF) goto reached_EOF;
          fprintf(outfile,"%s %s %s ",args[1],args[4],args[5]);fflush(outfile);
		  iline++;
        } /* coords_from_file */
      
      /* Switch on how many arguments are supplied. */
      /* Note that there are no 'breaks' after the cases, so these are entry points */
      switch(argv)
		{
		case 6 : strncpy(inbuff, args[5], MAXREAD);
          if ((rest=strchr(inbuff, ',')))     /* If it contains a comma */
            {
			  decdeg=2;                        /* Then not decimal degrees */
			  begin=inbuff;
              rest[0]='\0';                    /* Chop into sub string */
              rest++;                          /* Move to next substring */
			  ilon_deg=atoi(begin);
              begin=rest;
              if ((rest=strchr(begin, ',')))
                {
				  rest[0]='\0';
				  rest++;
                  ilon_min=atoi(begin);
                  ilon_sec=atoi(rest);
                } 
              else 
                {
                  ilon_min=0;
                  ilon_sec=0;
                }
            } 
          else 
            {
              decdeg=1;                        /* Else it's decimal */
			  longitude=atof(args[5]);
			}

        case 5 : strncpy(inbuff, args[4], MAXREAD);
          if ((rest=strchr(inbuff, ',')))
            {
              decdeg=2;
			  begin=inbuff;
              rest[0]='\0';
              rest++;
              ilat_deg=atoi(begin);
              begin=rest;
              if ((rest=strchr(begin, ',')))
				{
                  rest[0]='\0';
                  rest++;
                  ilat_min=atoi(begin);
                  ilat_sec=atoi(rest);
				}
              else 
				{
                  ilat_min=0;
                  ilat_sec=0;
                }
            }
          else 
            {
              decdeg=1;
			  latitude=atof(args[4]);
            }
          
//         case 4 : strncpy(inbuff, args[3], MAXREAD);
//           inbuff[0]=toupper(inbuff[0]);
//           if (inbuff[0]=='K') units=1;
//           else if (inbuff[0]=='M') units=2;
//           else if (inbuff[0]=='F') units=3;
//           if (strlen(inbuff)>1)
// 			{
//               inbuff[0]='\0';
//               begin=inbuff+1;
//               alt=atof(begin);
//             }
// 
//         case 3 : strncpy(inbuff, args[2], MAXREAD);
//           inbuff[0]=toupper(inbuff[0]);
// 		  if (inbuff[0]=='M') igdgc=1;    /* height is above  mean sea level*/
// 		  else if (inbuff[0]=='E') igdgc=2; /* height is above  WGS 84 ellepsoid */

          
        case 2 : strncpy(inbuff, args[1], MAXREAD);
          if ((rest=strchr(inbuff, '-')))   /* If it contains a dash */
            {
			  range = 2;                     /* They want a range */
              rest[0]='\0';                  /* Sep dates */
              rest++;
              begin=rest;
              if ((rest=strchr(begin, '-')))    /* If it contains 2 dashs */
                {
                  rest[0]='\0';                  /* Sep step */
				  rest++;
				  step=atof(rest);               /* Get step size */
                }
			  if ((rest=strchr(begin, ',')))    /* If it contains a comma */
				{
                  decyears=2;                    /* It's not decimal years */
                  rest[0]='\0';
                  rest++;
                  ieyear=atoi(begin);
                  begin=rest;
                  if ((rest=strchr(begin, ',')))
                    {
                      rest[0]='\0';
                      rest++;
                      iemonth=atoi(begin);
                      ieday=atoi(rest);
                    } 
                  else 
                    {
					  iemonth=0;
                      ieday=0;
                    }
                  if ((rest=strchr(inbuff, ',')))
                    {
                      begin=inbuff;
                      rest[0]='\0';
                      rest++;
                      isyear=atoi(begin);
                      begin=rest;
					  if ((rest=strchr(begin, ',')))
                        {
                          rest[0]='\0';
						  rest++;
                          ismonth=atoi(begin);
                          isday=atoi(rest);
                        } 
                      else 
                        {
						  ismonth=0;
                          isday=0;
                        }
                    } 
                  else 
					{
                      sdate=atof(inbuff);
                    }
                } 
              else 
                {
                  decyears=1;                    /* Else it's decimal years */
                  sdate=atof(inbuff);
                  edate=atof(begin);
                }
            } 
          else 
            {
              range = 1;
              if ((rest=strchr(inbuff, ',')))   /* If it contains a comma */
                {
                  decyears=2;                    /* It's not decimal years */
                  begin=inbuff;
                  rest[0]='\0';
                  rest++;
                  isyear=atoi(begin);
                  begin=rest;
                  if ((rest=strchr(begin, ',')))
                    {
                      rest[0]='\0';
                      rest++;
                      ismonth=atoi(begin);
                      isday=atoi(rest);
                    } 
                  else 
                    {
                      ismonth=0;
                      isday=0;
					}
                } 
              else 
				{
                  decyears=1;                    /* Else it's decimal years */
				  sdate=atof(args[1]);
                }
            }
          if (sdate==0)
            {                        /* If date not valid */
              decyears=-1;
              range=-1;
			}
			break;
//
		}

	  if (range == 2 && coords_from_file)
		{
		  printf("Error in line %1d, date = %s: date ranges not allowed for file option\n\n",iline,args[1]);
		  exit(2);
		}

	  /*  Obtain the desired model file and read the data  */

	 /* if date specified in command line then warn if past end of validity */

      /*  Take in field data  */
      
      /* Get date */
      
	  if (coords_from_file && !arg_err && (decyears != 1 && decyears != 2))
        {printf("\nError: unrecognized date %s in coordinate file line %1d\n\n",args[1],iline); arg_err = 1;}

	  if (coords_from_file && !arg_err && range != 1)
		{printf("\nError: unrecognized date %s in coordinate file line %1d\n\n",args[1],iline); arg_err = 1;}

	  if (coords_from_file && !arg_err && (sdate < minyr || sdate >maxyr))
		{printf("\nWarning:  date out of range in coordinate file line %1d\n\n",iline);
		printf("\nExpected range = %6.1lf - %6.1lf, entered %6.1lf\n",minyr,maxyr,sdate);}


	   /* Get altitude min and max for selected model. */
	  minalt= -10; /* To be defined */
	  maxalt= 1000;

	  /* Get Coordinate prefs */

// 	  if (coords_from_file && !arg_err && (igdgc != 1 && igdgc != 2))
// 		{printf("\nError: Unrecognized height reference %s in coordinate file line %1d.\n\n",args[1],iline); arg_err = 1;}

	   /* If needed modify height referencing */
// 	  if (igdgc==2)
// 		{
// 		  Geoid.UseGeoid = 0;     /* height above WGS-84 Ellipsoid */
// 		}else if (igdgc==1)
// 		{
// 		  Geoid.UseGeoid = 1;    /* height above MSL */
// 		}


// 	  /* Do unit conversions if neccessary */
// 	  if (units==2)
// 		{
// 		  minalt*=1000.0;
// 		  maxalt*=1000.0;
// 		}
// 	  else if (units==3)
// 		{
// 		  minalt*=3280.0839895;
// 		  maxalt*=3280.0839895;
// 		}

// 	  /* Get altitude */
// 
// 	  if (coords_from_file && !arg_err && (alt < minalt || alt > maxalt))
// 		{printf("\nError: unrecognized altitude %s in coordinate file line %1d\n\n",args[3],iline); arg_err = 1;}
// 
// 	  /* Convert altitude to km */
// 	  if (units==2)
// 		{
// 		  alt *= 0.001;
// 		}
// 	  else if (units==3)
// 		{
// 		  alt /= 3280.0839895;
// 		}



	  /* Get lat/long prefs */

	  if (coords_from_file && !arg_err && decdeg != 1 )
		{printf("\nError: unrecognized lat %s or lon %s in coordinate file line %1d\n\n",args[4],args[5],iline); arg_err = 1;}



	  /* Get lat/lon */


	  /** This will compute everything needed for 1 point in time. **/

			CoordGeodetic.lambda = longitude;
			CoordGeodetic.phi	 = latitude;
//			CoordGeodetic.HeightAboveGeoid = alt;
			UserDate.DecimalYear = sdate;


//			MAG_ConvertGeoidToEllipsoidHeight(&CoordGeodetic, &Geoid);   /*This converts the height above mean sea level to height above the WGS-84 ellipsoid*/
			//MAG_GeodeticToSpherical(Ellip, CoordGeodetic, &CoordSpherical);
//			alt = 108.0//CoordGeodetic.HeightAboveGeoid;
			epoch = (float) sdate;
			
			loadapxsh_("apexsh.dat", &epoch);
			apex_alt = 108.0; /*Has to be higher than 108.0*/
			flat = (float)latitude;//CoordSpherical.phig;
			flon = (float)longitude;//CoordSpherical.lambda;
			falt = 108.0;//(float)CoordGeodetic.HeightAboveGeoid;//CoordSpherical.r - Ellip.re;
			//apxg2q_(&flat, &flon, &falt, &vecflag, &maglat, &maglon, &ra1, &ra2, &ra3);
			apxg2all_(&flat, &flon, &falt, &apex_alt, &vecflag, &qlat, &qlon, &maglat, &maglon, &ra1, &ra2, &ra3, &ra4, &ra5, &ra6, &ra7, &ra8, &ra9, &ra10);
	  /** Above will compute everything for 1 point in time.  **/


	  /*  Output the final results. */


    if (coords_from_file)
		{
	print_result_file(outfile,
			maglat,
			maglon);
		}

	  if (coords_from_file)
		again = !feof(coordfile) && !arg_err;

	  if (again == 1)
		{
		  /* Reset defaults to catch on all while loops */
		  igdgc=decyears=units=decdeg=-1;
		  ismonth=isday=isyear=sdate=edate=range=step=-1;
		  latitude=ilat_deg=ilat_min=ilat_sec=200;
		  longitude=ilon_deg=ilon_min=ilon_sec=200;
		  alt=-9999999;
		  argv = 1;
		}
	} /* while (again == 1) */

 reached_EOF:

  if (coords_from_file) printf("\n Processed %1d lines\n\n",iline);

  if (coords_from_file && !feof(coordfile) && arg_err) printf("Terminated prematurely due to argument error in coordinate file\n\n");


fclose(coordfile);
fclose(outfile);

// if (Geoid.GeoidHeightBuffer)
// 	{
// 	free(Geoid.GeoidHeightBuffer);
// 	Geoid.GeoidHeightBuffer = NULL;
// 	}
  return 0;
}



void print_result_file(FILE *outf, double maglat, double maglon)
{
	fprintf(outf," %8.2lf %8.2lf\n",maglat, maglon);
	return;
} /* print_result_file */


/****************************************************************************/
/*                                                                          */
/*                       Subroutine safegets                                */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  Gets characters from stdin untill it has reached n characters or \n,    */
/*     whichever comes first.  \n is converted to \0.                       */
/*                                                                          */
/*  Input: n - Integer number of chars                                      */
/*         *buffer - Character array ptr which can contain n+1 characters   */
/*                                                                          */
/*  Output: size - integer size of sting in buffer                          */
/*                                                                          */
/*  Note: All strings will be null terminated.                              */
/*                                                                          */
/*  By: David Owens                                                         */
/*      dio@ngdc.noaa.gov                                                   */
/****************************************************************************/

int safegets(char *buffer,int n){
   char *ptr;                    /** ptr used for finding '\n' **/

   fgets(buffer,n,stdin);        /** Get n chars **/
   buffer[n+1]='\0';             /** Set last char to null **/
   ptr=strchr(buffer,'\n');      /** If string contains '\n' **/
   if (ptr!=NULL){                /** If string contains '\n' **/
      ptr[0]='\0';               /** Change char to '\0' **/
   }

   return strlen(buffer);        /** Return the length **/
}


/****************************************************************************/
/*                                                                          */
/*                       Subroutine degrees_to_decimal                      */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*     Converts degrees,minutes, seconds to decimal degrees.                */
/*                                                                          */
/*     Input:                                                               */
/*            degrees - Integer degrees                                     */
/*            minutes - Integer minutes                                     */
/*            seconds - Integer seconds                                     */
/*                                                                          */
/*     Output:                                                              */
/*            decimal - degrees in decimal degrees                          */
/*                                                                          */
/*     C                                                                    */
/*           C. H. Shaffer                                                  */
/*           Lockheed Missiles and Space Company, Sunnyvale CA              */
/*           August 12, 1988                                                */
/*                                                                          */
/****************************************************************************/

double degrees_to_decimal(int degrees,int minutes,int seconds)
{
   double deg;
   double min;
   double sec;
   double decimal;

   deg = degrees;
   min = minutes/60.0;
   sec = seconds/3600.0;

   decimal = fabs(sec) + fabs(min) + fabs(deg);

   if (deg < 0) {
      decimal = -decimal;
   } else if (deg == 0){
      if (min < 0){
         decimal = -decimal;
      } else if (min == 0){
         if (sec<0){
            decimal = -decimal;
         }
      }
   }

   return(decimal);
}

/****************************************************************************/
/*                                                                          */
/*                           Subroutine julday                              */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*     Computes the decimal day of year from month, day, year.              */
/*     Leap years accounted for 1900 and 2000 are not leap years.           */
/*                                                                          */
/*     Input:                                                               */
/*           year - Integer year of interest                                */
/*           month - Integer month of interest                              */
/*           day - Integer day of interest                                  */
/*                                                                          */
/*     Output:                                                              */
/*           date - Julian date to thousandth of year                       */
/*                                                                          */
/*     FORTRAN                                                              */
/*           S. McLean                                                      */
/*           NGDC, NOAA egc1, 325 Broadway, Boulder CO.  80301              */
/*                                                                          */
/*     C                                                                    */
/*           C. H. Shaffer                                                  */
/*           Lockheed Missiles and Space Company, Sunnyvale CA              */
/*           August 12, 1988                                                */
/*                                                                          */
/*     Julday Bug Fix                                                       */
/*           Thanks to Rob Raper                                            */
/****************************************************************************/


double julday(i_month, i_day, i_year)
int i_month;
int i_day;
int i_year;
{
   int   aggregate_first_day_of_month[13];
   int   leap_year = 0;
   int   truncated_dividend;
   double year;
   double day;
   double decimal_date;
   double remainder = 0.0;
   double divisor = 4.0;
   double dividend;
   double left_over;

   aggregate_first_day_of_month[1] = 1;
   aggregate_first_day_of_month[2] = 32;
   aggregate_first_day_of_month[3] = 60;
   aggregate_first_day_of_month[4] = 91;
   aggregate_first_day_of_month[5] = 121;
   aggregate_first_day_of_month[6] = 152;
   aggregate_first_day_of_month[7] = 182;
   aggregate_first_day_of_month[8] = 213;
   aggregate_first_day_of_month[9] = 244;
   aggregate_first_day_of_month[10] = 274;
   aggregate_first_day_of_month[11] = 305;
   aggregate_first_day_of_month[12] = 335;

   /* Test for leap year.  If true add one to day. */

   year = i_year;                                 /*    Century Years not   */
   if ((i_year != 1900) && (i_year != 2100))      /*  divisible by 400 are  */
   {                                              /*      NOT leap years    */
	  dividend = year/divisor;
	  truncated_dividend = dividend;
	  left_over = dividend - truncated_dividend;
	  remainder = left_over*divisor;
	  if ((remainder > 0.0) && (i_month > 2))
	  {
	 leap_year = 1;
	  }
	  else
	  {
	 leap_year = 0;
	  }
   }
   day = aggregate_first_day_of_month[i_month] + i_day - 1 + leap_year;
   if (leap_year)
   {
	  decimal_date = year + (day/366.0);  /*In version 3.0 this was incorrect*/
   }
   else
   {
	  decimal_date = year + (day/365.0);  /*In version 3.0 this was incorrect*/
   }
   return(decimal_date);
}



